// Copyright (c) Microsoft Corporation
// All rights reserved. 
//
// Licensed under the Apache License, Version 2.0 (the ""License""); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
//
// THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//
// See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.

#define NTDDI_VERSION 0x06010000

#include <v8.h>
#include <string>
#include <node_object_wrap.h>
#include <ppltasks.h>
#include "CollectionsConverter.h"
#include "CollectionsWrap.h"
#include "node-async.h"
#include "NodeRtUtils.h"
#include "OpaqueWrapper.h"
#include "WrapperBase.h"

#using <Windows.WinMD>

// this undefs fixes the issues of compiling Windows.Data.Json, Windows.Storag.FileProperties, and Windows.Stroage.Search
// Some of the node header files brings windows definitions with the same names as some of the WinRT methods
#undef DocumentProperties
#undef GetObject
#undef CreateEvent
#undef FindText
#undef SendMessage

const char* REGISTRATION_TOKEN_MAP_PROPERTY_NAME = "__registrationTokenMap__";

using namespace v8;
using namespace node;
using namespace concurrency;

namespace NodeRT { namespace Windows { namespace Media { 
  v8::Handle<v8::Value> WrapMediaControl(::Windows::Media::MediaControl^ wintRtInstance);
  ::Windows::Media::MediaControl^ UnwrapMediaControl(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIMediaMarker(::Windows::Media::IMediaMarker^ wintRtInstance);
  ::Windows::Media::IMediaMarker^ UnwrapIMediaMarker(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIMediaMarkers(::Windows::Media::IMediaMarkers^ wintRtInstance);
  ::Windows::Media::IMediaMarkers^ UnwrapIMediaMarkers(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMediaMarkerTypes(::Windows::Media::MediaMarkerTypes^ wintRtInstance);
  ::Windows::Media::MediaMarkerTypes^ UnwrapMediaMarkerTypes(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMusicDisplayProperties(::Windows::Media::MusicDisplayProperties^ wintRtInstance);
  ::Windows::Media::MusicDisplayProperties^ UnwrapMusicDisplayProperties(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapVideoDisplayProperties(::Windows::Media::VideoDisplayProperties^ wintRtInstance);
  ::Windows::Media::VideoDisplayProperties^ UnwrapVideoDisplayProperties(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapImageDisplayProperties(::Windows::Media::ImageDisplayProperties^ wintRtInstance);
  ::Windows::Media::ImageDisplayProperties^ UnwrapImageDisplayProperties(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapSystemMediaTransportControlsDisplayUpdater(::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ wintRtInstance);
  ::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ UnwrapSystemMediaTransportControlsDisplayUpdater(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapSystemMediaTransportControlsButtonPressedEventArgs(::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ wintRtInstance);
  ::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ UnwrapSystemMediaTransportControlsButtonPressedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapSystemMediaTransportControlsPropertyChangedEventArgs(::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ wintRtInstance);
  ::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ UnwrapSystemMediaTransportControlsPropertyChangedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapSystemMediaTransportControls(::Windows::Media::SystemMediaTransportControls^ wintRtInstance);
  ::Windows::Media::SystemMediaTransportControls^ UnwrapSystemMediaTransportControls(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIMediaExtension(::Windows::Media::IMediaExtension^ wintRtInstance);
  ::Windows::Media::IMediaExtension^ UnwrapIMediaExtension(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMediaExtensionManager(::Windows::Media::MediaExtensionManager^ wintRtInstance);
  ::Windows::Media::MediaExtensionManager^ UnwrapMediaExtensionManager(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapVideoEffects(::Windows::Media::VideoEffects^ wintRtInstance);
  ::Windows::Media::VideoEffects^ UnwrapVideoEffects(Handle<Value> value);
  


  static v8::Handle<v8::Value> InitSoundLevelEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("SoundLevel"), enumObject);

    enumObject->Set(String::NewSymbol("muted"), Integer::New(0));
    enumObject->Set(String::NewSymbol("low"), Integer::New(1));
    enumObject->Set(String::NewSymbol("full"), Integer::New(2));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitMediaPlaybackTypeEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("MediaPlaybackType"), enumObject);

    enumObject->Set(String::NewSymbol("unknown"), Integer::New(0));
    enumObject->Set(String::NewSymbol("music"), Integer::New(1));
    enumObject->Set(String::NewSymbol("video"), Integer::New(2));
    enumObject->Set(String::NewSymbol("image"), Integer::New(3));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitMediaPlaybackStatusEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("MediaPlaybackStatus"), enumObject);

    enumObject->Set(String::NewSymbol("closed"), Integer::New(0));
    enumObject->Set(String::NewSymbol("changing"), Integer::New(1));
    enumObject->Set(String::NewSymbol("stopped"), Integer::New(2));
    enumObject->Set(String::NewSymbol("playing"), Integer::New(3));
    enumObject->Set(String::NewSymbol("paused"), Integer::New(4));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitSystemMediaTransportControlsButtonEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("SystemMediaTransportControlsButton"), enumObject);

    enumObject->Set(String::NewSymbol("play"), Integer::New(0));
    enumObject->Set(String::NewSymbol("pause"), Integer::New(1));
    enumObject->Set(String::NewSymbol("stop"), Integer::New(2));
    enumObject->Set(String::NewSymbol("record"), Integer::New(3));
    enumObject->Set(String::NewSymbol("fastForward"), Integer::New(4));
    enumObject->Set(String::NewSymbol("rewind"), Integer::New(5));
    enumObject->Set(String::NewSymbol("next"), Integer::New(6));
    enumObject->Set(String::NewSymbol("previous"), Integer::New(7));
    enumObject->Set(String::NewSymbol("channelUp"), Integer::New(8));
    enumObject->Set(String::NewSymbol("channelDown"), Integer::New(9));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitSystemMediaTransportControlsPropertyEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("SystemMediaTransportControlsProperty"), enumObject);

    enumObject->Set(String::NewSymbol("soundLevel"), Integer::New(0));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitAudioProcessingEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("AudioProcessing"), enumObject);

    enumObject->Set(String::NewSymbol("default"), Integer::New(0));
    enumObject->Set(String::NewSymbol("raw"), Integer::New(1));

    return scope.Close(Undefined());
  }



  
  class MediaControl : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MediaControl"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->SetAccessor(String::NewSymbol("trackName"), TrackNameGetter, TrackNameSetter);
      constructor->SetAccessor(String::NewSymbol("isPlaying"), IsPlayingGetter, IsPlayingSetter);
      constructor->SetAccessor(String::NewSymbol("artistName"), ArtistNameGetter, ArtistNameSetter);
      constructor->SetAccessor(String::NewSymbol("albumArt"), AlbumArtGetter, AlbumArtSetter);
      constructor->SetAccessor(String::NewSymbol("soundLevel"), SoundLevelGetter);
      
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      constructor->Set(String::NewSymbol("addListener"), addListenerFunc);
      constructor->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      constructor->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      constructor->Set(String::NewSymbol("off"), removeListenerFunc);

      exports->Set(String::NewSymbol("MediaControl"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MediaControl(::Windows::Media::MediaControl^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaControl^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaControl^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaControl^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MediaControl *wrapperInstance = new MediaControl(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  




    static Handle<Value> TrackNameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaControl::TrackName;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void TrackNameSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      try 
      {

        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        ::Windows::Media::MediaControl::TrackName = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsPlayingGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        bool result = ::Windows::Media::MediaControl::IsPlaying;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsPlayingSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      try 
      {

        
        bool winRtValue = value->BooleanValue();

        ::Windows::Media::MediaControl::IsPlaying = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ArtistNameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaControl::ArtistName;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ArtistNameSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      try 
      {

        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        ::Windows::Media::MediaControl::ArtistName = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> AlbumArtGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        ::Windows::Foundation::Uri^ result = ::Windows::Media::MediaControl::AlbumArt;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Foundation", "Uri", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void AlbumArtSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      try 
      {

        
        ::Windows::Foundation::Uri^ winRtValue = dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(value));

        ::Windows::Media::MediaControl::AlbumArt = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> SoundLevelGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        ::Windows::Media::SoundLevel result = ::Windows::Media::MediaControl::SoundLevel;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    

    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"channelDownPressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::ChannelDownPressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"channelUpPressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::ChannelUpPressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"fastForwardPressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::FastForwardPressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"nextTrackPressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::NextTrackPressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"pausePressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::PausePressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"playPauseTogglePressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::PlayPauseTogglePressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"playPressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::PlayPressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"previousTrackPressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::PreviousTrackPressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"recordPressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::RecordPressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"rewindPressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::RewindPressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"soundLevelChanged", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::SoundLevelChanged::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"stopPressed", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Media::MediaControl::StopPressed::add(
            ref new ::Windows::Foundation::EventHandler<::Platform::Object^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"channelDownPressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"channelUpPressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"fastForwardPressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"nextTrackPressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"pausePressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"playPauseTogglePressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"playPressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"previousTrackPressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"recordPressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"rewindPressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"soundLevelChanged", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"stopPressed", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"channelDownPressed", str))
        {
          ::Windows::Media::MediaControl::ChannelDownPressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"channelUpPressed", str))
        {
          ::Windows::Media::MediaControl::ChannelUpPressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"fastForwardPressed", str))
        {
          ::Windows::Media::MediaControl::FastForwardPressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"nextTrackPressed", str))
        {
          ::Windows::Media::MediaControl::NextTrackPressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"pausePressed", str))
        {
          ::Windows::Media::MediaControl::PausePressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"playPauseTogglePressed", str))
        {
          ::Windows::Media::MediaControl::PlayPauseTogglePressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"playPressed", str))
        {
          ::Windows::Media::MediaControl::PlayPressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"previousTrackPressed", str))
        {
          ::Windows::Media::MediaControl::PreviousTrackPressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"recordPressed", str))
        {
          ::Windows::Media::MediaControl::RecordPressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"rewindPressed", str))
        {
          ::Windows::Media::MediaControl::RewindPressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"soundLevelChanged", str))
        {
          ::Windows::Media::MediaControl::SoundLevelChanged::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"stopPressed", str))
        {
          ::Windows::Media::MediaControl::StopPressed::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Media::MediaControl^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMediaControl(::Windows::Media::MediaControl^ wintRtInstance);
    friend ::Windows::Media::MediaControl^ UnwrapMediaControl(Handle<Value> value);
    friend bool IsMediaControlWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MediaControl::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMediaControl(::Windows::Media::MediaControl^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MediaControl::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaControl^ UnwrapMediaControl(Handle<Value> value)
  {
     return MediaControl::Unwrap<MediaControl>(value.As<Object>())->_instance;
  }

  void InitMediaControl(Handle<Object> exports)
  {
    MediaControl::Init(exports);
  }

  class IMediaMarker : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IMediaMarker"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("mediaMarkerType"), MediaMarkerTypeGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("text"), TextGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("time"), TimeGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IMediaMarker"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IMediaMarker(::Windows::Media::IMediaMarker^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::IMediaMarker^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::IMediaMarker^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::IMediaMarker^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IMediaMarker *wrapperInstance = new IMediaMarker(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> MediaMarkerTypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::IMediaMarker^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IMediaMarker *wrapper = IMediaMarker::Unwrap<IMediaMarker>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->MediaMarkerType;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> TextGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::IMediaMarker^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IMediaMarker *wrapper = IMediaMarker::Unwrap<IMediaMarker>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Text;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> TimeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::IMediaMarker^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IMediaMarker *wrapper = IMediaMarker::Unwrap<IMediaMarker>(info.This());

      try 
      {
        ::Windows::Foundation::TimeSpan result = wrapper->_instance->Time;
        return scope.Close(Number::New(result.Duration/10000.0));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Media::IMediaMarker^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIMediaMarker(::Windows::Media::IMediaMarker^ wintRtInstance);
    friend ::Windows::Media::IMediaMarker^ UnwrapIMediaMarker(Handle<Value> value);
    friend bool IsIMediaMarkerWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IMediaMarker::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIMediaMarker(::Windows::Media::IMediaMarker^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IMediaMarker::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::IMediaMarker^ UnwrapIMediaMarker(Handle<Value> value)
  {
     return IMediaMarker::Unwrap<IMediaMarker>(value.As<Object>())->_instance;
  }

  void InitIMediaMarker(Handle<Object> exports)
  {
    IMediaMarker::Init(exports);
  }

  class IMediaMarkers : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IMediaMarkers"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("markers"), MarkersGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IMediaMarkers"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IMediaMarkers(::Windows::Media::IMediaMarkers^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::IMediaMarkers^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::IMediaMarkers^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::IMediaMarkers^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IMediaMarkers *wrapperInstance = new IMediaMarkers(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> MarkersGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::IMediaMarkers^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IMediaMarkers *wrapper = IMediaMarkers::Unwrap<IMediaMarkers>(info.This());

      try 
      {
        ::Windows::Foundation::Collections::IVectorView<::Windows::Media::IMediaMarker^>^ result = wrapper->_instance->Markers;
        return scope.Close(NodeRT::Collections::VectorViewWrapper<::Windows::Media::IMediaMarker^>::CreateVectorViewWrapper(result, 
            [](::Windows::Media::IMediaMarker^ val) -> Handle<Value> {
              return WrapIMediaMarker(val);
            },
            [](Handle<Value> value) -> bool {
              return NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::IMediaMarker^>(value);
            },
            [](Handle<Value> value) -> ::Windows::Media::IMediaMarker^ {
              return UnwrapIMediaMarker(value);
            }
          ));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Media::IMediaMarkers^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIMediaMarkers(::Windows::Media::IMediaMarkers^ wintRtInstance);
    friend ::Windows::Media::IMediaMarkers^ UnwrapIMediaMarkers(Handle<Value> value);
    friend bool IsIMediaMarkersWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IMediaMarkers::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIMediaMarkers(::Windows::Media::IMediaMarkers^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IMediaMarkers::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::IMediaMarkers^ UnwrapIMediaMarkers(Handle<Value> value)
  {
     return IMediaMarkers::Unwrap<IMediaMarkers>(value.As<Object>())->_instance;
  }

  void InitIMediaMarkers(Handle<Object> exports)
  {
    IMediaMarkers::Init(exports);
  }

  class MediaMarkerTypes : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MediaMarkerTypes"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->SetAccessor(String::NewSymbol("bookmark"), BookmarkGetter);

      exports->Set(String::NewSymbol("MediaMarkerTypes"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MediaMarkerTypes(::Windows::Media::MediaMarkerTypes^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaMarkerTypes^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaMarkerTypes^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaMarkerTypes^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MediaMarkerTypes *wrapperInstance = new MediaMarkerTypes(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  




    static Handle<Value> BookmarkGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaMarkerTypes::Bookmark;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    

  private:
    ::Windows::Media::MediaMarkerTypes^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMediaMarkerTypes(::Windows::Media::MediaMarkerTypes^ wintRtInstance);
    friend ::Windows::Media::MediaMarkerTypes^ UnwrapMediaMarkerTypes(Handle<Value> value);
    friend bool IsMediaMarkerTypesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MediaMarkerTypes::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMediaMarkerTypes(::Windows::Media::MediaMarkerTypes^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MediaMarkerTypes::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaMarkerTypes^ UnwrapMediaMarkerTypes(Handle<Value> value)
  {
     return MediaMarkerTypes::Unwrap<MediaMarkerTypes>(value.As<Object>())->_instance;
  }

  void InitMediaMarkerTypes(Handle<Object> exports)
  {
    MediaMarkerTypes::Init(exports);
  }

  class MusicDisplayProperties : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MusicDisplayProperties"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("title"), TitleGetter, TitleSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("artist"), ArtistGetter, ArtistSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("albumArtist"), AlbumArtistGetter, AlbumArtistSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("MusicDisplayProperties"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MusicDisplayProperties(::Windows::Media::MusicDisplayProperties^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MusicDisplayProperties^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MusicDisplayProperties^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MusicDisplayProperties^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MusicDisplayProperties *wrapperInstance = new MusicDisplayProperties(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> TitleGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MusicDisplayProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MusicDisplayProperties *wrapper = MusicDisplayProperties::Unwrap<MusicDisplayProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Title;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void TitleSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MusicDisplayProperties^>(info.This()))
      {
        return;
      }

      MusicDisplayProperties *wrapper = MusicDisplayProperties::Unwrap<MusicDisplayProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Title = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ArtistGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MusicDisplayProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MusicDisplayProperties *wrapper = MusicDisplayProperties::Unwrap<MusicDisplayProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Artist;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ArtistSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MusicDisplayProperties^>(info.This()))
      {
        return;
      }

      MusicDisplayProperties *wrapper = MusicDisplayProperties::Unwrap<MusicDisplayProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Artist = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> AlbumArtistGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MusicDisplayProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MusicDisplayProperties *wrapper = MusicDisplayProperties::Unwrap<MusicDisplayProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->AlbumArtist;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void AlbumArtistSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MusicDisplayProperties^>(info.This()))
      {
        return;
      }

      MusicDisplayProperties *wrapper = MusicDisplayProperties::Unwrap<MusicDisplayProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->AlbumArtist = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Media::MusicDisplayProperties^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMusicDisplayProperties(::Windows::Media::MusicDisplayProperties^ wintRtInstance);
    friend ::Windows::Media::MusicDisplayProperties^ UnwrapMusicDisplayProperties(Handle<Value> value);
    friend bool IsMusicDisplayPropertiesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MusicDisplayProperties::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMusicDisplayProperties(::Windows::Media::MusicDisplayProperties^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MusicDisplayProperties::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MusicDisplayProperties^ UnwrapMusicDisplayProperties(Handle<Value> value)
  {
     return MusicDisplayProperties::Unwrap<MusicDisplayProperties>(value.As<Object>())->_instance;
  }

  void InitMusicDisplayProperties(Handle<Object> exports)
  {
    MusicDisplayProperties::Init(exports);
  }

  class VideoDisplayProperties : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("VideoDisplayProperties"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("title"), TitleGetter, TitleSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("subtitle"), SubtitleGetter, SubtitleSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("VideoDisplayProperties"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    VideoDisplayProperties(::Windows::Media::VideoDisplayProperties^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::VideoDisplayProperties^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::VideoDisplayProperties^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::VideoDisplayProperties^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      VideoDisplayProperties *wrapperInstance = new VideoDisplayProperties(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> TitleGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::VideoDisplayProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoDisplayProperties *wrapper = VideoDisplayProperties::Unwrap<VideoDisplayProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Title;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void TitleSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::VideoDisplayProperties^>(info.This()))
      {
        return;
      }

      VideoDisplayProperties *wrapper = VideoDisplayProperties::Unwrap<VideoDisplayProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Title = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> SubtitleGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::VideoDisplayProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoDisplayProperties *wrapper = VideoDisplayProperties::Unwrap<VideoDisplayProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Subtitle;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SubtitleSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::VideoDisplayProperties^>(info.This()))
      {
        return;
      }

      VideoDisplayProperties *wrapper = VideoDisplayProperties::Unwrap<VideoDisplayProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Subtitle = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Media::VideoDisplayProperties^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapVideoDisplayProperties(::Windows::Media::VideoDisplayProperties^ wintRtInstance);
    friend ::Windows::Media::VideoDisplayProperties^ UnwrapVideoDisplayProperties(Handle<Value> value);
    friend bool IsVideoDisplayPropertiesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> VideoDisplayProperties::s_constructorTemplate;

  v8::Handle<v8::Value> WrapVideoDisplayProperties(::Windows::Media::VideoDisplayProperties^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(VideoDisplayProperties::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::VideoDisplayProperties^ UnwrapVideoDisplayProperties(Handle<Value> value)
  {
     return VideoDisplayProperties::Unwrap<VideoDisplayProperties>(value.As<Object>())->_instance;
  }

  void InitVideoDisplayProperties(Handle<Object> exports)
  {
    VideoDisplayProperties::Init(exports);
  }

  class ImageDisplayProperties : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ImageDisplayProperties"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("title"), TitleGetter, TitleSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("subtitle"), SubtitleGetter, SubtitleSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ImageDisplayProperties"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ImageDisplayProperties(::Windows::Media::ImageDisplayProperties^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::ImageDisplayProperties^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::ImageDisplayProperties^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::ImageDisplayProperties^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ImageDisplayProperties *wrapperInstance = new ImageDisplayProperties(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> TitleGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::ImageDisplayProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageDisplayProperties *wrapper = ImageDisplayProperties::Unwrap<ImageDisplayProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Title;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void TitleSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::ImageDisplayProperties^>(info.This()))
      {
        return;
      }

      ImageDisplayProperties *wrapper = ImageDisplayProperties::Unwrap<ImageDisplayProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Title = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> SubtitleGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::ImageDisplayProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageDisplayProperties *wrapper = ImageDisplayProperties::Unwrap<ImageDisplayProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Subtitle;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SubtitleSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::ImageDisplayProperties^>(info.This()))
      {
        return;
      }

      ImageDisplayProperties *wrapper = ImageDisplayProperties::Unwrap<ImageDisplayProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Subtitle = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Media::ImageDisplayProperties^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapImageDisplayProperties(::Windows::Media::ImageDisplayProperties^ wintRtInstance);
    friend ::Windows::Media::ImageDisplayProperties^ UnwrapImageDisplayProperties(Handle<Value> value);
    friend bool IsImageDisplayPropertiesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ImageDisplayProperties::s_constructorTemplate;

  v8::Handle<v8::Value> WrapImageDisplayProperties(::Windows::Media::ImageDisplayProperties^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ImageDisplayProperties::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::ImageDisplayProperties^ UnwrapImageDisplayProperties(Handle<Value> value)
  {
     return ImageDisplayProperties::Unwrap<ImageDisplayProperties>(value.As<Object>())->_instance;
  }

  void InitImageDisplayProperties(Handle<Object> exports)
  {
    ImageDisplayProperties::Init(exports);
  }

  class SystemMediaTransportControlsDisplayUpdater : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("SystemMediaTransportControlsDisplayUpdater"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("clearAll"), FunctionTemplate::New(ClearAll)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("update"), FunctionTemplate::New(Update)->GetFunction());
      
            
      func = FunctionTemplate::New(CopyFromFileAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("copyFromFileAsync"), func);
      
                  
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("type"), TypeGetter, TypeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("thumbnail"), ThumbnailGetter, ThumbnailSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("appMediaId"), AppMediaIdGetter, AppMediaIdSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("imageProperties"), ImagePropertiesGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("musicProperties"), MusicPropertiesGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("videoProperties"), VideoPropertiesGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("SystemMediaTransportControlsDisplayUpdater"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    SystemMediaTransportControlsDisplayUpdater(::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::SystemMediaTransportControlsDisplayUpdater^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      SystemMediaTransportControlsDisplayUpdater *wrapperInstance = new SystemMediaTransportControlsDisplayUpdater(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> CopyFromFileAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(args.This());

      ::Windows::Foundation::IAsyncOperation<bool>^ op;
    

      if (args.Length() == 3
        && args[0]->IsInt32()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Storage::StorageFile^>(args[1]))
      {
        try
        {
          ::Windows::Media::MediaPlaybackType arg0 = static_cast<::Windows::Media::MediaPlaybackType>(args[0]->Int32Value());
          ::Windows::Storage::StorageFile^ arg1 = dynamic_cast<::Windows::Storage::StorageFile^>(NodeRT::Utils::GetObjectInstance(args[1]));
          
          op = wrapper->_instance->CopyFromFileAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<bool> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = Boolean::New(result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> ClearAll(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          wrapper->_instance->ClearAll();
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Update(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          wrapper->_instance->Update();
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> TypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(info.This());

      try 
      {
        ::Windows::Media::MediaPlaybackType result = wrapper->_instance->Type;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void TypeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(info.This());

      try 
      {
        
        ::Windows::Media::MediaPlaybackType winRtValue = static_cast<::Windows::Media::MediaPlaybackType>(value->Int32Value());

        wrapper->_instance->Type = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ThumbnailGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(info.This());

      try 
      {
        ::Windows::Storage::Streams::RandomAccessStreamReference^ result = wrapper->_instance->Thumbnail;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "RandomAccessStreamReference", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ThumbnailSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Storage::Streams::RandomAccessStreamReference^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(info.This());

      try 
      {
        
        ::Windows::Storage::Streams::RandomAccessStreamReference^ winRtValue = dynamic_cast<::Windows::Storage::Streams::RandomAccessStreamReference^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->Thumbnail = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> AppMediaIdGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->AppMediaId;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void AppMediaIdSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->AppMediaId = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ImagePropertiesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(info.This());

      try 
      {
        ::Windows::Media::ImageDisplayProperties^ result = wrapper->_instance->ImageProperties;
        return scope.Close(WrapImageDisplayProperties(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MusicPropertiesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(info.This());

      try 
      {
        ::Windows::Media::MusicDisplayProperties^ result = wrapper->_instance->MusicProperties;
        return scope.Close(WrapMusicDisplayProperties(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> VideoPropertiesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsDisplayUpdater^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsDisplayUpdater *wrapper = SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(info.This());

      try 
      {
        ::Windows::Media::VideoDisplayProperties^ result = wrapper->_instance->VideoProperties;
        return scope.Close(WrapVideoDisplayProperties(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapSystemMediaTransportControlsDisplayUpdater(::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ wintRtInstance);
    friend ::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ UnwrapSystemMediaTransportControlsDisplayUpdater(Handle<Value> value);
    friend bool IsSystemMediaTransportControlsDisplayUpdaterWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> SystemMediaTransportControlsDisplayUpdater::s_constructorTemplate;

  v8::Handle<v8::Value> WrapSystemMediaTransportControlsDisplayUpdater(::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(SystemMediaTransportControlsDisplayUpdater::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ UnwrapSystemMediaTransportControlsDisplayUpdater(Handle<Value> value)
  {
     return SystemMediaTransportControlsDisplayUpdater::Unwrap<SystemMediaTransportControlsDisplayUpdater>(value.As<Object>())->_instance;
  }

  void InitSystemMediaTransportControlsDisplayUpdater(Handle<Object> exports)
  {
    SystemMediaTransportControlsDisplayUpdater::Init(exports);
  }

  class SystemMediaTransportControlsButtonPressedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("SystemMediaTransportControlsButtonPressedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("button"), ButtonGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("SystemMediaTransportControlsButtonPressedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    SystemMediaTransportControlsButtonPressedEventArgs(::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      SystemMediaTransportControlsButtonPressedEventArgs *wrapperInstance = new SystemMediaTransportControlsButtonPressedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> ButtonGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsButtonPressedEventArgs *wrapper = SystemMediaTransportControlsButtonPressedEventArgs::Unwrap<SystemMediaTransportControlsButtonPressedEventArgs>(info.This());

      try 
      {
        ::Windows::Media::SystemMediaTransportControlsButton result = wrapper->_instance->Button;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapSystemMediaTransportControlsButtonPressedEventArgs(::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ wintRtInstance);
    friend ::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ UnwrapSystemMediaTransportControlsButtonPressedEventArgs(Handle<Value> value);
    friend bool IsSystemMediaTransportControlsButtonPressedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> SystemMediaTransportControlsButtonPressedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapSystemMediaTransportControlsButtonPressedEventArgs(::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(SystemMediaTransportControlsButtonPressedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ UnwrapSystemMediaTransportControlsButtonPressedEventArgs(Handle<Value> value)
  {
     return SystemMediaTransportControlsButtonPressedEventArgs::Unwrap<SystemMediaTransportControlsButtonPressedEventArgs>(value.As<Object>())->_instance;
  }

  void InitSystemMediaTransportControlsButtonPressedEventArgs(Handle<Object> exports)
  {
    SystemMediaTransportControlsButtonPressedEventArgs::Init(exports);
  }

  class SystemMediaTransportControlsPropertyChangedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("SystemMediaTransportControlsPropertyChangedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("property"), PropertyGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("SystemMediaTransportControlsPropertyChangedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    SystemMediaTransportControlsPropertyChangedEventArgs(::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      SystemMediaTransportControlsPropertyChangedEventArgs *wrapperInstance = new SystemMediaTransportControlsPropertyChangedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> PropertyGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControlsPropertyChangedEventArgs *wrapper = SystemMediaTransportControlsPropertyChangedEventArgs::Unwrap<SystemMediaTransportControlsPropertyChangedEventArgs>(info.This());

      try 
      {
        ::Windows::Media::SystemMediaTransportControlsProperty result = wrapper->_instance->Property;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapSystemMediaTransportControlsPropertyChangedEventArgs(::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ wintRtInstance);
    friend ::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ UnwrapSystemMediaTransportControlsPropertyChangedEventArgs(Handle<Value> value);
    friend bool IsSystemMediaTransportControlsPropertyChangedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> SystemMediaTransportControlsPropertyChangedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapSystemMediaTransportControlsPropertyChangedEventArgs(::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(SystemMediaTransportControlsPropertyChangedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ UnwrapSystemMediaTransportControlsPropertyChangedEventArgs(Handle<Value> value)
  {
     return SystemMediaTransportControlsPropertyChangedEventArgs::Unwrap<SystemMediaTransportControlsPropertyChangedEventArgs>(value.As<Object>())->_instance;
  }

  void InitSystemMediaTransportControlsPropertyChangedEventArgs(Handle<Object> exports)
  {
    SystemMediaTransportControlsPropertyChangedEventArgs::Init(exports);
  }

  class SystemMediaTransportControls : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("SystemMediaTransportControls"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                        
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("playbackStatus"), PlaybackStatusGetter, PlaybackStatusSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isStopEnabled"), IsStopEnabledGetter, IsStopEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isRewindEnabled"), IsRewindEnabledGetter, IsRewindEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isRecordEnabled"), IsRecordEnabledGetter, IsRecordEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isPreviousEnabled"), IsPreviousEnabledGetter, IsPreviousEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isPlayEnabled"), IsPlayEnabledGetter, IsPlayEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isPauseEnabled"), IsPauseEnabledGetter, IsPauseEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isNextEnabled"), IsNextEnabledGetter, IsNextEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isFastForwardEnabled"), IsFastForwardEnabledGetter, IsFastForwardEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isEnabled"), IsEnabledGetter, IsEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isChannelUpEnabled"), IsChannelUpEnabledGetter, IsChannelUpEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("isChannelDownEnabled"), IsChannelDownEnabledGetter, IsChannelDownEnabledSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("displayUpdater"), DisplayUpdaterGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("soundLevel"), SoundLevelGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("getForCurrentView"), FunctionTemplate::New(GetForCurrentView)->GetFunction());

      exports->Set(String::NewSymbol("SystemMediaTransportControls"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    SystemMediaTransportControls(::Windows::Media::SystemMediaTransportControls^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::SystemMediaTransportControls^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::SystemMediaTransportControls^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      SystemMediaTransportControls *wrapperInstance = new SystemMediaTransportControls(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  


    static Handle<Value> GetForCurrentView(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Media::SystemMediaTransportControls^ result;
          result = ::Windows::Media::SystemMediaTransportControls::GetForCurrentView();
          return scope.Close(WrapSystemMediaTransportControls(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }

    static Handle<Value> PlaybackStatusGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        ::Windows::Media::MediaPlaybackStatus result = wrapper->_instance->PlaybackStatus;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void PlaybackStatusSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        ::Windows::Media::MediaPlaybackStatus winRtValue = static_cast<::Windows::Media::MediaPlaybackStatus>(value->Int32Value());

        wrapper->_instance->PlaybackStatus = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsStopEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsStopEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsStopEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsStopEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsRewindEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsRewindEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsRewindEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsRewindEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsRecordEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsRecordEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsRecordEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsRecordEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsPreviousEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsPreviousEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsPreviousEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsPreviousEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsPlayEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsPlayEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsPlayEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsPlayEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsPauseEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsPauseEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsPauseEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsPauseEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsNextEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsNextEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsNextEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsNextEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsFastForwardEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsFastForwardEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsFastForwardEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsFastForwardEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsChannelUpEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsChannelUpEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsChannelUpEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsChannelUpEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IsChannelDownEnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        bool result = wrapper->_instance->IsChannelDownEnabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IsChannelDownEnabledSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return;
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->IsChannelDownEnabled = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> DisplayUpdaterGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        ::Windows::Media::SystemMediaTransportControlsDisplayUpdater^ result = wrapper->_instance->DisplayUpdater;
        return scope.Close(WrapSystemMediaTransportControlsDisplayUpdater(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SoundLevelGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(info.This());

      try 
      {
        ::Windows::Media::SoundLevel result = wrapper->_instance->SoundLevel;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"buttonPressed", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->ButtonPressed::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Media::SystemMediaTransportControls^, ::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^>(
            [callbackObjPtr](::Windows::Media::SystemMediaTransportControls^ arg0, ::Windows::Media::SystemMediaTransportControlsButtonPressedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapSystemMediaTransportControls(arg0);
                Handle<Value> wrappedArg1 = WrapSystemMediaTransportControlsButtonPressedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"propertyChanged", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->PropertyChanged::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Media::SystemMediaTransportControls^, ::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^>(
            [callbackObjPtr](::Windows::Media::SystemMediaTransportControls^ arg0, ::Windows::Media::SystemMediaTransportControlsPropertyChangedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapSystemMediaTransportControls(arg0);
                Handle<Value> wrappedArg1 = WrapSystemMediaTransportControlsPropertyChangedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"buttonPressed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"propertyChanged", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"buttonPressed", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(args.This());
          wrapper->_instance->ButtonPressed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"propertyChanged", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::SystemMediaTransportControls^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          SystemMediaTransportControls *wrapper = SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(args.This());
          wrapper->_instance->PropertyChanged::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Media::SystemMediaTransportControls^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapSystemMediaTransportControls(::Windows::Media::SystemMediaTransportControls^ wintRtInstance);
    friend ::Windows::Media::SystemMediaTransportControls^ UnwrapSystemMediaTransportControls(Handle<Value> value);
    friend bool IsSystemMediaTransportControlsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> SystemMediaTransportControls::s_constructorTemplate;

  v8::Handle<v8::Value> WrapSystemMediaTransportControls(::Windows::Media::SystemMediaTransportControls^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(SystemMediaTransportControls::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::SystemMediaTransportControls^ UnwrapSystemMediaTransportControls(Handle<Value> value)
  {
     return SystemMediaTransportControls::Unwrap<SystemMediaTransportControls>(value.As<Object>())->_instance;
  }

  void InitSystemMediaTransportControls(Handle<Object> exports)
  {
    SystemMediaTransportControls::Init(exports);
  }

  class IMediaExtension : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IMediaExtension"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("setProperties"), FunctionTemplate::New(SetProperties)->GetFunction());
      
                        
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IMediaExtension"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IMediaExtension(::Windows::Media::IMediaExtension^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::IMediaExtension^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::IMediaExtension^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::IMediaExtension^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IMediaExtension *wrapperInstance = new IMediaExtension(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> SetProperties(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::IMediaExtension^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      IMediaExtension *wrapper = IMediaExtension::Unwrap<IMediaExtension>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Collections::IPropertySet^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Collections::IPropertySet^ arg0 = dynamic_cast<::Windows::Foundation::Collections::IPropertySet^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          wrapper->_instance->SetProperties(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }





  private:
    ::Windows::Media::IMediaExtension^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIMediaExtension(::Windows::Media::IMediaExtension^ wintRtInstance);
    friend ::Windows::Media::IMediaExtension^ UnwrapIMediaExtension(Handle<Value> value);
    friend bool IsIMediaExtensionWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IMediaExtension::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIMediaExtension(::Windows::Media::IMediaExtension^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IMediaExtension::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::IMediaExtension^ UnwrapIMediaExtension(Handle<Value> value)
  {
     return IMediaExtension::Unwrap<IMediaExtension>(value.As<Object>())->_instance;
  }

  void InitIMediaExtension(Handle<Object> exports)
  {
    IMediaExtension::Init(exports);
  }

  class MediaExtensionManager : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MediaExtensionManager"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("registerSchemeHandler"), FunctionTemplate::New(RegisterSchemeHandler)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("registerByteStreamHandler"), FunctionTemplate::New(RegisterByteStreamHandler)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("registerAudioDecoder"), FunctionTemplate::New(RegisterAudioDecoder)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("registerAudioEncoder"), FunctionTemplate::New(RegisterAudioEncoder)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("registerVideoDecoder"), FunctionTemplate::New(RegisterVideoDecoder)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("registerVideoEncoder"), FunctionTemplate::New(RegisterVideoEncoder)->GetFunction());
      
                        
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("MediaExtensionManager"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MediaExtensionManager(::Windows::Media::MediaExtensionManager^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaExtensionManager^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaExtensionManager^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaExtensionManager^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Media::MediaExtensionManager();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MediaExtensionManager *wrapperInstance = new MediaExtensionManager(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> RegisterSchemeHandler(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaExtensionManager^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaExtensionManager *wrapper = MediaExtensionManager::Unwrap<MediaExtensionManager>(args.This());

      if (args.Length() == 2
        && args[0]->IsString()
        && args[1]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          wrapper->_instance->RegisterSchemeHandler(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && args[0]->IsString()
        && args[1]->IsString()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Collections::IPropertySet^>(args[2]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          ::Windows::Foundation::Collections::IPropertySet^ arg2 = dynamic_cast<::Windows::Foundation::Collections::IPropertySet^>(NodeRT::Utils::GetObjectInstance(args[2]));
          
          wrapper->_instance->RegisterSchemeHandler(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RegisterByteStreamHandler(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaExtensionManager^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaExtensionManager *wrapper = MediaExtensionManager::Unwrap<MediaExtensionManager>(args.This());

      if (args.Length() == 3
        && args[0]->IsString()
        && args[1]->IsString()
        && args[2]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          Platform::String^ arg2 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[2])));
          
          wrapper->_instance->RegisterByteStreamHandler(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 4
        && args[0]->IsString()
        && args[1]->IsString()
        && args[2]->IsString()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Collections::IPropertySet^>(args[3]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          Platform::String^ arg2 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[2])));
          ::Windows::Foundation::Collections::IPropertySet^ arg3 = dynamic_cast<::Windows::Foundation::Collections::IPropertySet^>(NodeRT::Utils::GetObjectInstance(args[3]));
          
          wrapper->_instance->RegisterByteStreamHandler(arg0, arg1, arg2, arg3);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RegisterAudioDecoder(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaExtensionManager^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaExtensionManager *wrapper = MediaExtensionManager::Unwrap<MediaExtensionManager>(args.This());

      if (args.Length() == 3
        && args[0]->IsString()
        && NodeRT::Utils::IsGuid(args[1])
        && NodeRT::Utils::IsGuid(args[2]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Platform::Guid arg1 = NodeRT::Utils::GuidFromJs(args[1]);
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          
          wrapper->_instance->RegisterAudioDecoder(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 4
        && args[0]->IsString()
        && NodeRT::Utils::IsGuid(args[1])
        && NodeRT::Utils::IsGuid(args[2])
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Collections::IPropertySet^>(args[3]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Platform::Guid arg1 = NodeRT::Utils::GuidFromJs(args[1]);
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          ::Windows::Foundation::Collections::IPropertySet^ arg3 = dynamic_cast<::Windows::Foundation::Collections::IPropertySet^>(NodeRT::Utils::GetObjectInstance(args[3]));
          
          wrapper->_instance->RegisterAudioDecoder(arg0, arg1, arg2, arg3);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RegisterAudioEncoder(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaExtensionManager^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaExtensionManager *wrapper = MediaExtensionManager::Unwrap<MediaExtensionManager>(args.This());

      if (args.Length() == 3
        && args[0]->IsString()
        && NodeRT::Utils::IsGuid(args[1])
        && NodeRT::Utils::IsGuid(args[2]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Platform::Guid arg1 = NodeRT::Utils::GuidFromJs(args[1]);
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          
          wrapper->_instance->RegisterAudioEncoder(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 4
        && args[0]->IsString()
        && NodeRT::Utils::IsGuid(args[1])
        && NodeRT::Utils::IsGuid(args[2])
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Collections::IPropertySet^>(args[3]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Platform::Guid arg1 = NodeRT::Utils::GuidFromJs(args[1]);
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          ::Windows::Foundation::Collections::IPropertySet^ arg3 = dynamic_cast<::Windows::Foundation::Collections::IPropertySet^>(NodeRT::Utils::GetObjectInstance(args[3]));
          
          wrapper->_instance->RegisterAudioEncoder(arg0, arg1, arg2, arg3);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RegisterVideoDecoder(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaExtensionManager^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaExtensionManager *wrapper = MediaExtensionManager::Unwrap<MediaExtensionManager>(args.This());

      if (args.Length() == 3
        && args[0]->IsString()
        && NodeRT::Utils::IsGuid(args[1])
        && NodeRT::Utils::IsGuid(args[2]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Platform::Guid arg1 = NodeRT::Utils::GuidFromJs(args[1]);
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          
          wrapper->_instance->RegisterVideoDecoder(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 4
        && args[0]->IsString()
        && NodeRT::Utils::IsGuid(args[1])
        && NodeRT::Utils::IsGuid(args[2])
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Collections::IPropertySet^>(args[3]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Platform::Guid arg1 = NodeRT::Utils::GuidFromJs(args[1]);
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          ::Windows::Foundation::Collections::IPropertySet^ arg3 = dynamic_cast<::Windows::Foundation::Collections::IPropertySet^>(NodeRT::Utils::GetObjectInstance(args[3]));
          
          wrapper->_instance->RegisterVideoDecoder(arg0, arg1, arg2, arg3);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RegisterVideoEncoder(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaExtensionManager^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaExtensionManager *wrapper = MediaExtensionManager::Unwrap<MediaExtensionManager>(args.This());

      if (args.Length() == 3
        && args[0]->IsString()
        && NodeRT::Utils::IsGuid(args[1])
        && NodeRT::Utils::IsGuid(args[2]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Platform::Guid arg1 = NodeRT::Utils::GuidFromJs(args[1]);
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          
          wrapper->_instance->RegisterVideoEncoder(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 4
        && args[0]->IsString()
        && NodeRT::Utils::IsGuid(args[1])
        && NodeRT::Utils::IsGuid(args[2])
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Collections::IPropertySet^>(args[3]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Platform::Guid arg1 = NodeRT::Utils::GuidFromJs(args[1]);
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          ::Windows::Foundation::Collections::IPropertySet^ arg3 = dynamic_cast<::Windows::Foundation::Collections::IPropertySet^>(NodeRT::Utils::GetObjectInstance(args[3]));
          
          wrapper->_instance->RegisterVideoEncoder(arg0, arg1, arg2, arg3);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }





  private:
    ::Windows::Media::MediaExtensionManager^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMediaExtensionManager(::Windows::Media::MediaExtensionManager^ wintRtInstance);
    friend ::Windows::Media::MediaExtensionManager^ UnwrapMediaExtensionManager(Handle<Value> value);
    friend bool IsMediaExtensionManagerWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MediaExtensionManager::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMediaExtensionManager(::Windows::Media::MediaExtensionManager^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MediaExtensionManager::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaExtensionManager^ UnwrapMediaExtensionManager(Handle<Value> value)
  {
     return MediaExtensionManager::Unwrap<MediaExtensionManager>(value.As<Object>())->_instance;
  }

  void InitMediaExtensionManager(Handle<Object> exports)
  {
    MediaExtensionManager::Init(exports);
  }

  class VideoEffects : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("VideoEffects"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->SetAccessor(String::NewSymbol("videoStabilization"), VideoStabilizationGetter);

      exports->Set(String::NewSymbol("VideoEffects"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    VideoEffects(::Windows::Media::VideoEffects^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::VideoEffects^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::VideoEffects^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::VideoEffects^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      VideoEffects *wrapperInstance = new VideoEffects(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  




    static Handle<Value> VideoStabilizationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::VideoEffects::VideoStabilization;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    

  private:
    ::Windows::Media::VideoEffects^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapVideoEffects(::Windows::Media::VideoEffects^ wintRtInstance);
    friend ::Windows::Media::VideoEffects^ UnwrapVideoEffects(Handle<Value> value);
    friend bool IsVideoEffectsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> VideoEffects::s_constructorTemplate;

  v8::Handle<v8::Value> WrapVideoEffects(::Windows::Media::VideoEffects^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(VideoEffects::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::VideoEffects^ UnwrapVideoEffects(Handle<Value> value)
  {
     return VideoEffects::Unwrap<VideoEffects>(value.As<Object>())->_instance;
  }

  void InitVideoEffects(Handle<Object> exports)
  {
    VideoEffects::Init(exports);
  }

} } } 

void init(Handle<Object> exports)
{
  if (FAILED(CoInitializeEx(nullptr, COINIT_MULTITHREADED)))
  {
    ThrowException(v8::Exception::Error(NodeRT::Utils::NewString(L"error in CoInitializeEx()")));
    return;
  }
  
  NodeRT::Windows::Media::InitSoundLevelEnum(exports);
  NodeRT::Windows::Media::InitMediaPlaybackTypeEnum(exports);
  NodeRT::Windows::Media::InitMediaPlaybackStatusEnum(exports);
  NodeRT::Windows::Media::InitSystemMediaTransportControlsButtonEnum(exports);
  NodeRT::Windows::Media::InitSystemMediaTransportControlsPropertyEnum(exports);
  NodeRT::Windows::Media::InitAudioProcessingEnum(exports);
  NodeRT::Windows::Media::InitMediaControl(exports);
  NodeRT::Windows::Media::InitIMediaMarker(exports);
  NodeRT::Windows::Media::InitIMediaMarkers(exports);
  NodeRT::Windows::Media::InitMediaMarkerTypes(exports);
  NodeRT::Windows::Media::InitMusicDisplayProperties(exports);
  NodeRT::Windows::Media::InitVideoDisplayProperties(exports);
  NodeRT::Windows::Media::InitImageDisplayProperties(exports);
  NodeRT::Windows::Media::InitSystemMediaTransportControlsDisplayUpdater(exports);
  NodeRT::Windows::Media::InitSystemMediaTransportControlsButtonPressedEventArgs(exports);
  NodeRT::Windows::Media::InitSystemMediaTransportControlsPropertyChangedEventArgs(exports);
  NodeRT::Windows::Media::InitSystemMediaTransportControls(exports);
  NodeRT::Windows::Media::InitIMediaExtension(exports);
  NodeRT::Windows::Media::InitMediaExtensionManager(exports);
  NodeRT::Windows::Media::InitVideoEffects(exports);

  NodeRT::Utils::RegisterNameSpace("Windows.Media", exports);
}


NODE_MODULE(binding, init)